
{******************************************}
{                                          }
{             FastReport v3.0              }
{         EasyTable enduser components     }
{                                          }

// Copyright (c) AidAim Software
// http://www.aidaim.com
// 22 December 2006

{                                          }
{******************************************}

unit frxTETComponents;

interface

{$I frx.inc}

uses
  Windows, Classes, frxClass, frxCustomDB, DB, EasyTable
{$IFDEF Delphi6}
, Variants
{$ENDIF}
{$IFDEF QBUILDER}
, fqbClass
{$ENDIF};


type
  TfrxTETComponents = class(TfrxDBComponents)
  private
    FDefaultDatabase: TEasyDatabase;
    FOldComponents: TfrxTETComponents;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetDescription: String; override;
  published
    property DefaultDatabase: TEasyDatabase read FDefaultDatabase write FDefaultDatabase;
  end;

  TfrxTETDatabase = class(TfrxCustomDatabase)
  private
    FDatabase: TEasyDatabase;
  protected
    procedure SetConnected(Value: Boolean); override;
    procedure SetDatabaseName(const Value: String); override;
    procedure SetLoginPrompt(Value: Boolean); override;
    procedure SetParams(Value: TStrings); override;
    function GetConnected: Boolean; override;
    function GetDatabaseName: String; override;
    function GetLoginPrompt: Boolean; override;
    function GetParams: TStrings; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    class function GetDescription: String; override;
    procedure SetLogin(const Login, Password: String); override;
    property Database: TEasyDatabase read FDatabase;
  published
    property DatabaseName;
    property LoginPrompt;
    property Params;
    property Connected;
  end;

  TfrxTETTable = class(TfrxCustomTable)
  private
    FDatabase: TfrxTETDatabase;
    FTable: TEasyTable;
    procedure SetDatabase(const Value: TfrxTETDatabase);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure SetMaster(const Value: TDataSource); override;
    procedure SetMasterFields(const Value: String); override;
    procedure SetIndexFieldNames(const Value: String); override;
    procedure SetIndexName(const Value: String); override;
    procedure SetTableName(const Value: String); override;
    function GetIndexFieldNames: String; override;
    function GetIndexName: String; override;
    function GetTableName: String; override;
  public
    constructor Create(AOwner: TComponent); override;
    constructor DesignCreate(AOwner: TComponent; Flags: Word); override;
    class function GetDescription: String; override;
    procedure BeforeStartReport; override;
    property Table: TEasyTable read FTable;
  published
    property Database: TfrxTETDatabase read FDatabase write SetDatabase;
  end;

  TfrxTETQuery = class(TfrxCustomQuery)
  private
    FDatabase: TfrxTETDatabase;
    FQuery: TEasyQuery;
    procedure SetDatabase(const Value: TfrxTETDatabase);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure SetMaster(const Value: TDataSource); override;
    procedure SetSQL(Value: TStrings); override;
    function GetSQL: TStrings; override;
  public
    constructor Create(AOwner: TComponent); override;
    constructor DesignCreate(AOwner: TComponent; Flags: Word); override;
    class function GetDescription: String; override;
    procedure BeforeStartReport; override;
    procedure UpdateParams; override;
{$IFDEF QBUILDER}
    function QBEngine: TfqbEngine; override;
{$ENDIF}
    property Query: TEasyQuery read FQuery;
  published
    property Database: TfrxTETDatabase read FDatabase write SetDatabase;
  end;

{$IFDEF QBUILDER}
  TfrxEngineTET = class(TfqbEngine)
  private
    FQuery: TEasyQuery;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ReadTableList(ATableList: TStrings); override;
    procedure ReadFieldList(const ATableName: string; var AFieldList: TfqbFieldList); override;
    function ResultDataSet: TDataSet; override;
    procedure SetSQL(const Value: string); override;
  end;
{$ENDIF}


var
  TETComponents: TfrxTETComponents;


implementation

{$R *.res}

uses 
  frxTETRTTI, 
{$IFNDEF NO_EDITORS}
  frxTETEditor, 
{$ENDIF}
  frxDsgnIntf, frxRes;


{ TfrxTETComponents }

constructor TfrxTETComponents.Create(AOwner: TComponent);
begin
  inherited;
  FOldComponents := TETComponents;
  TETComponents := Self;
end;

destructor TfrxTETComponents.Destroy;
begin
  if TETComponents = Self then
    TETComponents := FOldComponents;
  inherited;
end;

function TfrxTETComponents.GetDescription: String;
begin
  Result := 'TET';
end;


{ TfrxTETDatabase }

constructor TfrxTETDatabase.Create(AOwner: TComponent);
begin
  inherited;
  FDatabase := TEasyDatabase.Create(nil);
  Component := FDatabase;
end;

destructor TfrxTETDatabase.Destroy;
begin
  inherited;
end;

class function TfrxTETDatabase.GetDescription: String;
begin
  Result := 'TET Database';
end;

function TfrxTETDatabase.GetConnected: Boolean;
begin
  Result := FDatabase.Connected;
end;

function TfrxTETDatabase.GetDatabaseName: String;
begin
  Result := FDatabase.DatabaseName;
end;

function TfrxTETDatabase.GetLoginPrompt: Boolean;
begin
  Result := False;
//  Result := FDatabase.LoginPrompt;
end;

function TfrxTETDatabase.GetParams: TStrings;
begin
  Result := nil;
  //Result := FDatabase.Params;
end;

procedure TfrxTETDatabase.SetConnected(Value: Boolean);
begin
  BeforeConnect(Value);
  FDatabase.Connected := Value;
end;

procedure TfrxTETDatabase.SetDatabaseName(const Value: String);
begin
  FDatabase.DatabaseName := Value;
end;

procedure TfrxTETDatabase.SetLoginPrompt(Value: Boolean);
begin
//  FDatabase.LoginPrompt := Value;
end;

procedure TfrxTETDatabase.SetParams(Value: TStrings);
begin
//  FDatabase.Params := Value;
end;

procedure TfrxTETDatabase.SetLogin(const Login, Password: String);
begin
// this method is used by "New connection" wizard
// for example (IBX):
// Params.Text := 'user_name=' + Login + #13#10 + 'password=' + Password;
end;


{ TfrxTETTable }

constructor TfrxTETTable.Create(AOwner: TComponent);
begin
  FTable := TEasyTable.Create(nil);
  DataSet := FTable;
  SetDatabase(nil);
  inherited;
end;

constructor TfrxTETTable.DesignCreate(AOwner: TComponent; Flags: Word);
var
  i: Integer;
  l: TList;
begin
  inherited;
  l := Report.AllObjects;
  for i := 0 to l.Count - 1 do
    if TObject(l[i]) is TfrxTETDatabase then
    begin
      SetDatabase(TfrxTETDatabase(l[i]));
      break;
    end;
end;

class function TfrxTETTable.GetDescription: String;
begin
  Result := 'TET Table';
end;

procedure TfrxTETTable.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = FDatabase) then
    SetDatabase(nil);
end;

procedure TfrxTETTable.SetDatabase(const Value: TfrxTETDatabase);
begin
  FDatabase := Value;
  if Value <> nil then
    FTable.DatabaseName := Value.DatabaseName
  else if TETComponents <> nil then
    FTable.DatabaseName := TETComponents.DefaultDatabase.DatabaseName
  else
    FTable.DatabaseName := '';
end;

function TfrxTETTable.GetIndexFieldNames: String;
begin
  Result := FTable.IndexFieldNames;
end;

function TfrxTETTable.GetIndexName: String;
begin
  Result := FTable.IndexName;
end;

function TfrxTETTable.GetTableName: String;
begin
  Result := FTable.TableName;
end;

procedure TfrxTETTable.SetIndexFieldNames(const Value: String);
begin
  FTable.IndexFieldNames := Value;
end;

procedure TfrxTETTable.SetIndexName(const Value: String);
begin
  FTable.IndexName := Value;
end;

procedure TfrxTETTable.SetTableName(const Value: String);
begin
  FTable.TableName := Value;
end;

procedure TfrxTETTable.SetMaster(const Value: TDataSource);
begin
  FTable.MasterSource := Value;
end;

procedure TfrxTETTable.SetMasterFields(const Value: String);
begin
  FTable.MasterFields := Value;
end;

procedure TfrxTETTable.BeforeStartReport;
begin
  SetDatabase(FDatabase);
end;


{ TfrxTETQuery }

constructor TfrxTETQuery.Create(AOwner: TComponent);
begin
  FQuery := TEasyQuery.Create(nil);
  Dataset := FQuery;
  SetDatabase(nil);
  inherited;
end;

constructor TfrxTETQuery.DesignCreate(AOwner: TComponent; Flags: Word);
var
  i: Integer;
  l: TList;
begin
  inherited;
  l := Report.AllObjects;
  for i := 0 to l.Count - 1 do
    if TObject(l[i]) is TfrxTETDatabase then
    begin
      SetDatabase(TfrxTETDatabase(l[i]));
      break;
    end;
end;

class function TfrxTETQuery.GetDescription: String;
begin
  Result := 'TET Query';
end;

procedure TfrxTETQuery.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = FDatabase) then
    SetDatabase(nil);
end;

procedure TfrxTETQuery.SetDatabase(const Value: TfrxTETDatabase);
begin
  FDatabase := Value;
  if Value <> nil then
    FQuery.DatabaseName := Value.DatabaseName
  else if TETComponents <> nil then
    FQuery.DatabaseName := TETComponents.DefaultDatabase.DatabaseName
  else
    FQuery.DatabaseName := '';
end;

function TfrxTETQuery.GetSQL: TStrings;
begin
  Result := FQuery.SQL;
end;

procedure TfrxTETQuery.SetSQL(Value: TStrings);
begin
  FQuery.SQL := Value;
end;

procedure TfrxTETQuery.SetMaster(const Value: TDataSource);
begin
  FQuery.DataSource := Value;
end;

procedure TfrxTETQuery.UpdateParams;
begin
  frxParamsToTParams(Self, FQuery.Params);
end;

procedure TfrxTETQuery.BeforeStartReport;
begin
  SetDatabase(FDatabase);
end;

{$IFDEF QBUILDER}
function TfrxTETQuery.QBEngine: TfqbEngine;
begin
  Result := TfrxEngineTET.Create(nil);
  TfrxEngineTET(Result).FQuery.Database := FQuery.Database;
end;
{$ENDIF}


{$IFDEF QBUILDER}
constructor TfrxEngineTET.Create(AOwner: TComponent);
begin
  inherited;
  FQuery := TEasyQuery.Create(Self);
end;

destructor TfrxEngineTET.Destroy;
begin
  FQuery.Free;
  inherited;
end;

procedure TfrxEngineTET.ReadFieldList(const ATableName: string;
  var AFieldList: TfqbFieldList);
var
  TempTable: TEasyTable;
  Fields: TFieldDefs;
  i: Integer;
  tmpField: TfqbField;
begin
  AFieldList.Clear;
  TempTable := TEasyTable.Create(Self);
  TempTable.Database := FQuery.Database;
  TempTable.TableName := ATableName;
  Fields := TempTable.FieldDefs;
  try
    try
      TempTable.Active := True;
      tmpField:= TfqbField(AFieldList.Add);
      tmpField.FieldName := '*';
      for i := 0 to Fields.Count - 1 do
      begin
        tmpField := TfqbField(AFieldList.Add);
        tmpField.FieldName := Fields.Items[i].Name;
        tmpField.FieldType := Ord(Fields.Items[i].DataType)
      end;
    except
    end;
  finally
    TempTable.Free;
  end;
end;

procedure TfrxEngineTET.ReadTableList(ATableList: TStrings);
begin
  ATableList.Clear;
  FQuery.Database.GetTableNames(ATableList, ShowSystemTables);
end;

function TfrxEngineTET.ResultDataSet: TDataSet;
begin
  Result := FQuery;
end;

procedure TfrxEngineTET.SetSQL(const Value: string);
begin
  FQuery.SQL.Text := Value;
end;
{$ENDIF}


initialization
  frxObjects.RegisterObject1(TfrxTETDataBase, nil, 'EasyTable Database', 'TET', 0, 37);
  frxObjects.RegisterObject1(TfrxTETTable, nil, 'EasyTable Table', 'TET', 0, 38);
  frxObjects.RegisterObject1(TfrxTETQuery, nil, 'EasyTable Query', 'TET', 0, 39);

finalization
//  CatBmp.Free;
  frxObjects.UnRegister(TfrxTETDataBase);
  frxObjects.UnRegister(TfrxTETTable);
  frxObjects.UnRegister(TfrxTETQuery);


end.